## This is the plugin to handle appletalk laserwriters under
## tkchooser2. It has no access to global variables, only
## procedures and functions
##
## Ethan Gold <etgold@cs.vassar.edu> 2/24/98
##

## must provide a name, public name, and icon in addition to
## start and stop procedures

## these don't work as globals, only for
## non-procedural initialization

if {[debug]} { puts "loading lwplug..." }

## register the plugin with the main chooser
register_plugin appletalk lwplug LaserWriters

########## required functions #################

#### required procedures that simply return plugin information
proc lwplug.geticon {} { return "laserwriter.pnm" }
proc lwplug.getpubname {} { return "LaserWriters" }
proc lwplug.getprotocol {} { return "appletalk" }
## not required for API - using as a global variable
proc lwplug.afterdelay {} { return "120000" }

## start function - required
proc lwplug.start {} {
    if {[debug]} { puts "started LaserWriter Plugin" }

    set entlabel [plug_list_label]
    $entlabel configure -text "LaserWriters:"
    update
    ## make sure there aren't any undeleted widgets
    lwplug.widgets stop
    lwplug.widgets start
    lwplug.refresh
}

## stop funtion - required
proc lwplug.stop {} {
    if {[debug]} { puts "stopped LaserWriter Plugin" }
    lwplug.clearentities
    set ID [get_glob afterID]
    if {[string compare $ID ""] != 0 } {
	after cancel $ID
    }
    lwplug.widgets stop
}

## function to call when a listitem is double-clicked
proc lwplug.doubleclick {} {
    if {[debug]} { puts "lwplug: got double-click with [get_curr_item]" }
    ## create a window with papstatus information in it
    lwplug.showprinterstatus [get_curr_item] [appletalk.getcurrzone]
}

proc lwplug.newzone {} {
    set plugframe [plug_frame]
    $plugframe.status configure -text "scanning [appletalk.getcurrzone]..."
    update
    lwplug.clearentities
    lwplug.showentities [lwplug.getnames]
    $plugframe.status configure -text "ready."
}

############# End required functions ##################

## procedure to show the status of a printer
proc lwplug.showprinterstatus {printer zone} {

    ## make sure the name we use in the printer window path
    ## variable doesn't have any spaces.
    regsub -all " " $printer "" prnt

    set status "waiting for reply..."
    set w .lwplug_status_$prnt
    
    ## replace the current window for this printer
    ## if it exists
    destroy $w
    
    toplevel $w -class Dialog
    label $w.label -text "$printer@$zone" -relief groove
    message $w.mesg -width 300 -text $status
    button $w.close -text "Close" \
	    -command "destroy $w" -default active

    pack $w.label -side top -fill x
    pack $w.mesg -side top -fill x
    pack $w.close -side left -anchor s
    update

    bind $w <Return> "$w.close invoke"

    ## get the status
    set status [lwplug.getprinterstatus $printer $zone]
    $w.mesg configure -text $status
    focus $w
}

## function to get and return the status of a printer
proc lwplug.getprinterstatus {printer zone} {
    global appletalkdir

    set plugframe [plug_frame]
    $plugframe.status configure -text "querying $printer..."
    update

    set statfd [open "|$appletalkdir/bin/papstatus -p \"$printer@$zone\"" r]
    set status ""
    while {[gets $statfd line] != -1} {
	if {[string compare $status ""] == 0} {
	    set status $line
	} else {set status "$status\n$line"}
    }
    if {[string compare $status ""] == 0} {
	set status "no response."
    }
    $plugframe.status configure -text "ready."
    return $status
}

## function to do tilde completion on a filename
proc lwplug.tilde {name} {
    global env
    ## do tilde expansion
    regsub "~/" $name "$env(HOME)/" newfile
    regsub $env(USER) $env(HOME) "" tophome
    regsub "~" $newfile $tophome file_name
    return $file_name
}

proc lwplug.refresh {} {
    lwplug.newzone
    set_glob afterID [ after [lwplug.afterdelay] {lwplug.refresh}]
}

## procedure to call appletalk print function
proc lwplug.printfile {filterflag} {
    set thefile [get_glob "file"]
    if {[string compare $thefile ""] == 0} {
	puts "lwplug: no file specified"
	return
    }
    set printer "[get_curr_item]\:LaserWriter\@[appletalk.getcurrzone]"
    set filename [lwplug.tilde $thefile]
    puts "lwplug: trying to print $filename to $printer"
    set result [appletalk.print $printer $filename $filterflag]
    if {[debug]} { puts "lwplug: print result: $result" }
}

## procedure to preview a file to be printed
proc lwplug.previewfile {filterflag} {
    global printpreviewer printfilterfile
    if {[string compare $printpreviewer "" ] == 0 || !$filterflag} {
	set printpreviewer "xterm -e more"
    }
    set thefile [get_glob "file"]
    if {[string compare $thefile ""] == 0} {
	puts "lwplug: no file specified"
	return
    }
    set filename [lwplug.tilde $thefile]
    if {$filterflag} {
	if {[file readable $printfilterfile]} {
	    ## read in filter command
	    set filtfd [open "$printfilterfile" r]
	    gets $filtfd filtcmd
	    close $filtfd
	    if {[debug]} {
		puts "lwplug: opening |$filtcmd $filename | $printpreviewer"
	    }
	    set result ""
	    catch {eval exec $filtcmd $filename | $printpreviewer} result
	    if {[debug]} {puts "lwplug: $result"}
	} else {
	    error "LaserWriter" "Could not find the printfilter file \
		    $printfilterfile. Please uncheck the filter \
		    option and try again."
	    return
	}
    } else {
	catch {eval exec $printpreviewer $filename } result
    }
}

## procedure to build and destroy lwplug widgets
proc lwplug.widgets {command} {
    set plugframe [plug_frame]

    set filterflag 0
    set_glob file ""
    if {[string compare $command "start"] == 0} {
	checkbutton $plugframe.filter -variable filterflag -text "use filter"
	button $plugframe.print -text "Print File" \
		-command "lwplug.printfile \$filterflag"
	button $plugframe.preview -text "Preview File" \
		-command "lwplug.previewfile \$filterflag"
	label $plugframe.label -text "File:"
	message $plugframe.filename -width 200 -relief sunken -text ""
	pack $plugframe.label
	pack $plugframe.filename -fill x
	pack $plugframe.filter
	pack $plugframe.preview
	pack $plugframe.print
	$plugframe.status configure -text "ready."

	bind $plugframe.filename <ButtonRelease> {
	    fileselect lwplug.getfilename
	}

    } else {
	destroy $plugframe.print
	destroy $plugframe.preview
	destroy $plugframe.filter
	destroy $plugframe.filename
	destroy $plugframe.label
    }
    
}

## procedure to get a filename, set it globally, and list
## it in the filename widget
proc lwplug.getfilename {filename} {
    set plugframe [plug_frame]
    set_glob file "[pwd]/$filename"
    $plugframe.filename configure -text [get_glob "file"]
    if {[debug]} { puts "lwplug: file was [get_glob file]" }
}

## function to return the laserwriters in the current zone
proc lwplug.getnames {} {
    return [appletalk.namesfromlkup [lwplug.getentities]]
}

proc lwplug.getentities {} {
    set currzone [appletalk.getcurrzone]
    set entity "=:LaserWriter@$currzone"
    set results [appletalk.nbplkup $entity]
    set results [appletalk.namesfromlkup $results]
    return $results
}

## function to display passed in entities in
## the plugin's entity list frame
proc lwplug.showentities {entitylist} {
    set lbox [plug_list]
    foreach entity $entitylist {
	$lbox insert end $entity
    }
}

proc lwplug.clearentities {} {
    set lbox [plug_list]
    $lbox delete 0 end
}

if {[debug]} { puts "finished loading lwplug." }
